/**
 * Copyright (C) 2012 Jorge Jimenez (jorge@iryoku.com). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the copyright holders.
 *
 * This is my attempt to port the the GAUSSIAN shader by Boulotaur2024 to ReShade.
 * Some settings from the original are missing and I have added some other settings to achieve certain looks.
 * More info can be found at http://reshade.me/forum/shader-presentation/27-gaussian-blur-bloom-unsharpmask
 */

#include EFFECT_CONFIG(Ioxa)

#if USE_GAUSS

#pragma message "Gaussian Blur by Jorge Jimenez, Boulotaur2024 (ported by Ioxa)\n"

#if USE_Blur == 1
#define Use_GaussianBlur 1
#endif
#if USE_Sharpening == 1 
#define Use_Unsharpmask 1
#endif
#if USE_Bloom == 1
#define Use_GaussianBloom 1 
#endif

 /**
 * Copyright (C) 2012 Jorge Jimenez (jorge@iryoku.com). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are 
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the copyright holders.
 */

/* This is my attempt to port the the GAUSSIAN shader by Boulotaur2024 to ReShade.
   Some settings from the original are missing and I have added some other settings to achieve certain looks.
   More info can be found at 
   http://reshade.me/forum/shader-presentation/27-gaussian-blur-bloom-unsharpmask */ 

#define CoefLuma_G            float3(0.2126, 0.7152, 0.0722)      // BT.709 & sRBG luma coefficient (Monitors and HD Television)
#define sharp_strength_luma_G (CoefLuma_G * SharpStrength)
#define sharp_clampG        0.035

#define Gpx_size (ReShade::PixelSize*BlurOffset)

#if Use_GaussianBlur == 1
#if GaussTexScale == 1 
		#define txsize 0.5 
#elif GaussTexScale == 2 
		#define txsize 0.25 
#else 
		#define txsize 1 
#endif

texture GBlurTex2Dping{ Width = BUFFER_WIDTH*txsize; Height = BUFFER_HEIGHT*txsize; Format = RGBA8; };
sampler2D GBlurSamplerPing { Texture = GBlurTex2Dping; MinFilter = Anisotropic; MagFilter = Anisotropic; MaxAnisotropy = 16; MipFilter = Linear; AddressU = Clamp; SRGBTexture = FALSE;};
texture GBlurTex2Dpong{ Width = BUFFER_WIDTH*txsize; Height = BUFFER_HEIGHT*txsize; Format = RGBA8; };
sampler2D GBlurSamplerPong { Texture = GBlurTex2Dpong; MinFilter = Anisotropic; MagFilter = Anisotropic; MaxAnisotropy = 16; MipFilter = Linear; AddressU = Clamp; SRGBTexture = FALSE;};
#endif

#if Use_Unsharpmask == 1
#if SharpTexScale == 1 
		#define StxsizeW (BUFFER_WIDTH*0.5) 
		#define StxsizeH (BUFFER_HEIGHT*0.5)
#elif SharpTexScale == 2 
		#define StxsizeW (BUFFER_WIDTH*0.25) 
		#define StxsizeH (BUFFER_HEIGHT*0.25) 
#else 
		#define StxsizeW (BUFFER_WIDTH) 
		#define StxsizeH (BUFFER_HEIGHT) 
#endif

#define Spx_size (ReShade::PixelSize*UnsharpmaskOffset)

texture SBlurTex2Dping{ Width = StxsizeW; Height = StxsizeH; Format = RGBA8; };
sampler2D SBlurSamplerPing { Texture = SBlurTex2Dping; MinFilter = Linear; MagFilter = Linear; MipFilter = Linear; AddressU = Clamp; SRGBTexture = FALSE;};
texture SBlurTex2Dpong{ Width = StxsizeW; Height = StxsizeH; Format = RGBA8; };
sampler2D SBlurSamplerPong { Texture = SBlurTex2Dpong; MinFilter = Linear; MagFilter = Linear; MipFilter = Linear; AddressU = Clamp; SRGBTexture = FALSE;};
#endif

#if Use_GaussianBloom == 1
#if BloomTexScale == 1 
	#define Btxsize 0.5 
#elif BloomTexScale == 2 
	#define Btxsize 0.25 
#else 
	#define Btxsize 1 
#endif

texture BBlurTex2Dping{ Width = BUFFER_WIDTH*Btxsize; Height = BUFFER_HEIGHT*Btxsize; Format = RGBA8; };
texture BBlurTex2Dpong{ Width = BUFFER_WIDTH*Btxsize; Height = BUFFER_HEIGHT*Btxsize; Format = RGBA8; };
sampler2D BBlurSamplerPing { Texture = BBlurTex2Dping; MinFilter = Linear; MagFilter = Linear; MipFilter = Linear; AddressU = Clamp; SRGBTexture = FALSE;};
sampler2D BBlurSamplerPong { Texture = BBlurTex2Dpong; MinFilter = Linear; MagFilter = Linear; MipFilter = Linear; AddressU = Clamp; SRGBTexture = FALSE;};

texture BBlurTexScaleping{ Width = BUFFER_WIDTH*Btxsize*0.25; Height = BUFFER_HEIGHT*Btxsize*0.25; Format = RGBA8; };
texture BBlurTexScalepong{ Width = BUFFER_WIDTH*Btxsize*0.25; Height = BUFFER_HEIGHT*Btxsize*0.25; Format = RGBA8; };
sampler2D BBlurSamplerScalePing { Texture = BBlurTexScaleping; MinFilter = Linear; MagFilter = Linear; MipFilter = Linear; AddressU = Clamp; SRGBTexture = FALSE;};
sampler2D BBlurSamplerScalePong { Texture = BBlurTexScalepong; MinFilter = Linear; MagFilter = Linear; MipFilter = Linear; AddressU = Clamp; SRGBTexture = FALSE;};

#endif

float gDoBias(float b, float x)
{
	return x = pow(max(0,x), log(b)/log(0.5));
}

float gDoGain(float a, float b, float x)
{
	return x = lerp(gDoBias(1-a,2*x)*0.5,1-gDoBias(1-b,2-2*x)*0.5,step(0.5,x));
}

float3 gDoStep(float edge0, float edge1, float3 x)
{
	return clamp((x-edge0)/(edge1-edge0),0,1);
}

float3 gDoAcceleration(float edge0, float edge1, float x)
{
	x = pow(max(0,x),2);
	return gDoStep(edge0,edge1,x);
}

float3 gDoDeceleration(float edge0, float edge1, float3 x)
{
	x = 1-pow(max(0,1-x),2);
	return gDoStep(edge0,edge1,x);
}

float gDoSmoothstep(float edge0, float edge1, float x)
{
	x = clamp((x-edge0)/(edge1-edge0),0,1);
	return x*x*(3-2*x);
}

float gDoSmootherstep(float edge0, float edge1, float x)
{
	x = clamp((x-edge0)/(edge1-edge0),0,1);
	return x*x*x*(x*(x*6-15)+10);
}

float gDoSmoothererstep(float edge0, float edge1, float x)
{
	x = clamp((x-edge0)/(edge1-edge0),0,1);
	return x*x*x*x*(x*(x*(70-20*x)-84)+35);
}

#if Use_GaussianBlur == 1

//compute Sigma 

#define Sigma1 0.39894*exp(-0.5*0*0/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma2 0.39894*exp(-0.5*1*1/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma3 0.39894*exp(-0.5*2*2/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma4 0.39894*exp(-0.5*3*3/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma5 0.39894*exp(-0.5*4*4/(GaussSigma*GaussSigma))/GaussSigma

#if GaussSigma >= 2
#define Sigma6 0.39894*exp(-0.5*5*5/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma7 0.39894*exp(-0.5*6*6/(GaussSigma*GaussSigma))/GaussSigma
#endif

#if GaussSigma >= 3
#define Sigma8 0.39894*exp(-0.5*7*7/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma9 0.39894*exp(-0.5*8*8/(GaussSigma*GaussSigma))/GaussSigma
#endif 

#if GaussSigma >= 4
#define Sigma10 0.39894*exp(-0.5*9*9/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma11 0.39894*exp(-0.5*10*10/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma12 0.39894*exp(-0.5*11*11/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma13 0.39894*exp(-0.5*12*12/(GaussSigma*GaussSigma))/GaussSigma
#endif 

#if GaussSigma >= 5
#define Sigma14 0.39894*exp(-0.5*13*13/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma15 0.39894*exp(-0.5*14*14/(GaussSigma*GaussSigma))/GaussSigma
#endif

#if GaussSigma >= 6
#define Sigma16 0.39894*exp(-0.5*15*15/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma17 0.39894*exp(-0.5*17*17/(GaussSigma*GaussSigma))/GaussSigma
#endif 

#if GaussSigma >= 8
#define Sigma18 0.39894*exp(-0.5*17*17/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma19 0.39894*exp(-0.5*18*18/(GaussSigma*GaussSigma))/GaussSigma
#endif 

#if GaussSigma >= 9
#define Sigma20 0.39894*exp(-0.5*19*19/(GaussSigma*GaussSigma))/GaussSigma
#define Sigma21 0.39894*exp(-0.5*20*20/(GaussSigma*GaussSigma))/GaussSigma
#endif

//#define SigmaNorm 0.5/(Sigma1 + Sigma2 + Sigma3 + Sigma4 + Sigma5 + Sigma6 + Sigma7 + Sigma8 + Sigma9)
#define SigmaNorm 1.0

#define GaussWeight1 (Sigma1)
#define GaussOffset1 (0.0)

#define GaussWeight2 (Sigma2 + Sigma3)
#define GaussOffset2 ((Sigma2*1.0)+(Sigma3*2.0))/GaussWeight2

#define GaussWeight3 (Sigma4 + Sigma5 )
#define GaussOffset3 ((Sigma4*3.0)+(Sigma5*4.0))/GaussWeight3

#if GaussSigma >= 2
#define GaussWeight4 (Sigma6 + Sigma7)
#define GaussOffset4 ((Sigma6*5.0)+(Sigma7*6.0))/GaussWeight4
#else 
#define GaussWeight4 1.0
#define GaussOffset4 1.0
#endif

#if GaussSigma >= 3
#define GaussWeight5 (Sigma8 + Sigma9)
#define GaussOffset5 ((Sigma8*7.0)+(Sigma9*8.0))/GaussWeight5
#else 
#define GaussWeight5 1.0
#define GaussOffset5 1.0
#endif

#if GaussSigma >= 4
#define GaussWeight6 (Sigma10 + Sigma11)
#define GaussOffset6 ((Sigma10*9.0)+(Sigma11*10.0))/GaussWeight6

#define GaussWeight7 (Sigma12 + Sigma13)
#define GaussOffset7 ((Sigma12*11.0)+(Sigma13*12.0))/GaussWeight7
#else 
#define GaussWeight6 1.0
#define GaussOffset6 1.0

#define GaussWeight7 1.0
#define GaussOffset7 1.0
#endif 

#if GaussSigma >= 5
#define GaussWeight8 (Sigma14 + Sigma15)
#define GaussOffset8 ((Sigma14*13.0)+(Sigma15*14.0))/GaussWeight8
#else 
#define GaussWeight8 1.0
#define GaussOffset8 1.0
#endif

#if GaussSigma >= 6
#define GaussWeight9 (Sigma16 + Sigma17)
#define GaussOffset9 ((Sigma16*15.0)+(Sigma17*16.0))/GaussWeight9
#else 
#define GaussWeight9 1.0
#define GaussOffset9 1.0
#endif

#if GaussSigma >= 8
#define GaussWeight10 (Sigma18 + Sigma19)
#define GaussOffset10 ((Sigma18*17.0)+(Sigma19*18.0))/GaussWeight10
#else 
#define GaussWeight10 1.0
#define GaussOffset10 1.0
#endif

#if GaussSigma >= 9
#define GaussWeight11 (Sigma20 + Sigma21)
#define GaussOffset11 ((Sigma20*19.0)+(Sigma21*20.0))/GaussWeight11
#else 
#define GaussWeight11 1.0
#define GaussOffset11 1.0
#endif

#if GaussSigma == 3
	uniform const int N = 5;
#elif GaussSigma == 4 
	uniform const int N = 7;
#elif GaussSigma == 5 
	uniform const int N = 8;
#elif GaussSigma == 6 
	uniform const int N = 9;
#elif GaussSigma == 7 
	uniform const int N = 9;
#elif GaussSigma == 8 
	uniform const int N = 10;
#elif GaussSigma == 9 
	uniform const int N = 11;
#elif GaussSigma == 10 
	uniform const int N = 11;
#elif GaussSigma == 2 
	uniform const int N = 4;
#else 
	uniform const int N = 3;
#endif

//#define GaussWeightNorm 0.5/(GaussWeight1 + GaussWeight2 + GaussWeight3 + GaussWeight4 + GaussWeight5) //0.5804515482
#define GaussWeightNorm 1.0
//#define NNN GaussSigma
//#if NNN <= 3
	//#define NN 5
//#else
	//#define NN GaussSigma+2
//#endif


float3 normpdf4(in float3 v)
{
	float sigma = EdgeStrength;
	//v = dot(v,v);
	//return 0.39894*exp(-0.5*dot(v,v)/(sigma*sigma))/sigma;
	//return (exp(-(pow(v,2.0))/(2.0*pow(sigma,2.0))));
	//return (exp(-(v*v)/(2.0*sigma*sigma)));
	//return lerp(1.0/sigma,sign(v)/v,step(sigma,abs(v)));
	return pow(1/pow(1+(pow(v/sigma,2.0)),0.5),1.0);

}

float4 TexScale(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	return tex2D(ReShade::BackBuffer,texcoord);
}

float4 BlurTexScale(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
		return tex2D(ReShade::BackBuffer,texcoord);
}

float4 HGaussianBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	#if GaussTexScale == 0
		#define GaussSampler ReShade::BackBuffer
	#else 
		#define GaussSampler GBlurSamplerPing
	#endif

	//float4 color = tex2D(GaussSampler, texcoord);
	
	float sampleOffsets[11] = { GaussOffset1, GaussOffset2, GaussOffset3, GaussOffset4, GaussOffset5, GaussOffset6, GaussOffset7, GaussOffset8, GaussOffset9, GaussOffset10, GaussOffset11 };
	float sampleWeights[11] = { GaussWeight1, GaussWeight2, GaussWeight3, GaussWeight4, GaussWeight5, GaussWeight6, GaussWeight7, GaussWeight8, GaussWeight9, GaussWeight10, GaussWeight11 };
	//int N = 5;
	
	#if Use_SurfaceBlur == 1
		//float luma = dot(tex2D(ReShade::BackBuffer,texcoord).rgb,0.333);
		float3 luma = tex2D(ReShade::BackBuffer,texcoord).rgb;
		//luma = 0.0;
		float3 Color = tex2D(GaussSampler,texcoord).rgb;
		//float3 Color = 0.0;
		float3 factor = sampleWeights[0];
		float3 Z = factor;
		float3 final_colour = Color*factor;
	#else
		float4 color = tex2D(GaussSampler, texcoord);
		float factor = sampleWeights[0];
		float Z = sampleWeights[0];
		float3 finalColor = color.rgb * factor;
	#endif

	[loop]
	for(int i = 1; i < N; ++i) {
		#if Use_SurfaceBlur == 1
			Color = tex2D(GaussSampler, texcoord + float2(sampleOffsets[i] * Gpx_size.x, 0.0)).rgb;
			factor = normpdf4((Color-luma)) * sampleWeights[i];
			//factor = (exp(-(pow(Color-luma,2.0))/(2.0*pow(EdgeStrength,2.0)))) * sampleWeights[i];
			//factor = sampleWeights[i];
			Z += factor;
			final_colour += factor*Color;
			Color = tex2D(GaussSampler, texcoord - float2(sampleOffsets[i] * Gpx_size.x, 0.0)).rgb;
			factor = normpdf4((Color-luma)) * sampleWeights[i];
			//factor = (exp(-(pow(Color-luma,2.0))/(2.0*pow(EdgeStrength,2.0)))) * sampleWeights[i];
			//factor = sampleWeights[i];
			Z += factor;
			final_colour += factor*Color;
		#else
			color = tex2D(GaussSampler, texcoord + float2(sampleOffsets[i] * Gpx_size.x, 0.0));
			factor = sampleWeights[i];
			Z += factor;
			finalColor += color.rgb*factor;
			color = tex2D(GaussSampler, texcoord - float2(sampleOffsets[i] * Gpx_size.x, 0.0)); 
			factor = sampleWeights[i];
			Z += factor;
			finalColor += color.rgb*factor;
		#endif
	}
	
	#if Use_SurfaceBlur == 1
		//float4 color = float4(final_colour/Z,1.0,1.0,1.0);
		float4 color = float4(final_colour/Z,1.0);
	#else 
		color = float4(finalColor/1.0,1.0);
	#endif
	
	return color;
}

float4 VGaussianBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{

	float sampleOffsets[11] = { GaussOffset1, GaussOffset2, GaussOffset3, GaussOffset4, GaussOffset5, GaussOffset6, GaussOffset7, GaussOffset8, GaussOffset9, GaussOffset10, GaussOffset11 };
	float sampleWeights[11] = { GaussWeight1, GaussWeight2, GaussWeight3, GaussWeight4, GaussWeight5, GaussWeight6, GaussWeight7, GaussWeight8, GaussWeight9, GaussWeight10, GaussWeight11 };
	
	#if Use_SurfaceBlur == 1
		//float luma = dot(tex2D(ReShade::BackBuffer,texcoord).rgb,0.333);
		float3 luma = tex2D(ReShade::BackBuffer,texcoord).rgb;
		//luma = 0.0;
		float3 Color = tex2D(GBlurSamplerPong,texcoord).rgb;
		//float3 Color = 0.0;
		float3 factor = sampleWeights[0];
		float3 Z = factor;
		float3 final_colour = Color*factor;
	#else
		float4 color = tex2D(GBlurSamplerPong, texcoord);
		float factor = sampleWeights[0];
		float Z = sampleWeights[0];
		float3 finalColor = color.rgb * factor;
	#endif
		[loop]
	for(int j = 1; j < N; ++j) {
		#if Use_SurfaceBlur == 1
			Color = tex2D(GBlurSamplerPong, texcoord + float2(0.0, sampleOffsets[j] * Gpx_size.y)).rgb;
			factor = normpdf4((Color-luma)) * sampleWeights[j];
			//factor = (exp(-(pow(Color-luma,2.0))/(2.0*pow(EdgeStrength,2.0)))) * sampleWeights[j];
			//factor = sampleWeights[j];
			Z += factor;
			final_colour += factor*Color;
			Color = tex2D(GBlurSamplerPong, texcoord - float2(0.0, sampleOffsets[j] * Gpx_size.y)).rgb;
			factor = normpdf4((Color-luma)) * sampleWeights[j];
			//factor = (exp(-(pow(Color-luma,2.0))/(2.0*pow(EdgeStrength,2.0)))) * sampleWeights[j];
			//factor = sampleWeights[j];
			Z += factor;
			final_colour += factor*Color;
		#else
			color = tex2D(GBlurSamplerPong, texcoord + float2(0.0, sampleOffsets[j] * Gpx_size.y));
			factor = sampleWeights[j];
			Z += factor;
			finalColor += color.rgb*factor;
			color = tex2D(GBlurSamplerPong, texcoord - float2(0.0, sampleOffsets[j] * Gpx_size.y));
			factor = sampleWeights[j];
			Z += factor;
			finalColor += color.rgb*factor;
		#endif
	}
	 
	#if Use_SurfaceBlur == 1
		//float4 color = float4(final_colour/Z,1.0,1.0,1.0);
		float4 color = float4(final_colour/Z,1.0);
	#else 
		color = float4(finalColor/1.0,1.0);
	#endif
	
	return color;
}

float4 H2GaussianBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{

	float sampleOffsets[11] = { GaussOffset1, GaussOffset2, GaussOffset3, GaussOffset4, GaussOffset5, GaussOffset6, GaussOffset7, GaussOffset8, GaussOffset9, GaussOffset10, GaussOffset11 };
	float sampleWeights[11] = { GaussWeight1, GaussWeight2, GaussWeight3, GaussWeight4, GaussWeight5, GaussWeight6, GaussWeight7, GaussWeight8, GaussWeight9, GaussWeight10, GaussWeight11 };
	
	#if Use_SurfaceBlur == 1
		//float luma = dot(tex2D(ReShade::BackBuffer,texcoord).rgb,0.333);
		float3 luma = tex2D(ReShade::BackBuffer,texcoord).rgb;
		float3 Color;
		float3 factor;
		float3 Z = 0.0;
		float3 final_colour = 0.0;
	#else
		float4 color = tex2D(GBlurSamplerPing, texcoord) * sampleWeights[0];
	#endif
		[loop]
	for(int j = 1; j < N; ++j) {
		#if Use_SurfaceBlur == 1
			Color = tex2D(GBlurSamplerPing, texcoord + float2(sampleOffsets[j] * Gpx_size.x, 0.0)).rgb;
			factor = normpdf4((Color-luma)) * sampleWeights[j];
			//factor = (exp(-(pow(Color-luma,2.0))/(2.0*pow(EdgeStrength,2.0)))) * sampleWeights[j];
			Z += factor;
			final_colour += factor*Color;
			Color = tex2D(GBlurSamplerPing, texcoord - float2(sampleOffsets[j] * Gpx_size.x, 0.0)).rgb;
			factor = normpdf4((Color-luma)) * sampleWeights[j];
			//factor = (exp(-(pow(Color-luma,2.0))/(2.0*pow(EdgeStrength,2.0)))) * sampleWeights[j];
			Z += factor;
			final_colour += factor*Color;
		#else
			color += tex2D(GBlurSamplerPing, texcoord + float2(sampleOffsets[j] * Gpx_size.x, 0.0)) * sampleWeights[j];
			color += tex2D(GBlurSamplerPing, texcoord - float2(sampleOffsets[j] * Gpx_size.x, 0.0)) * sampleWeights[j];
		#endif
	}
	 
	#if Use_SurfaceBlur == 1
		//float4 color = float4(final_colour/Z,1.0,1.0,1.0);
		float4 color = float4(final_colour/Z,1.0);
	#endif

	return color;
}

float4 GaussianBlurFinalPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	
	float sampleOffsets[11] = { GaussOffset1, GaussOffset2, GaussOffset3, GaussOffset4, GaussOffset5, GaussOffset6, GaussOffset7, GaussOffset8, GaussOffset9, GaussOffset10, GaussOffset11 };
	float sampleWeights[11] = { GaussWeight1, GaussWeight2, GaussWeight3, GaussWeight4, GaussWeight5, GaussWeight6, GaussWeight7, GaussWeight8, GaussWeight9, GaussWeight10, GaussWeight11 };
	
	float4 orig = tex2D(ReShade::BackBuffer, texcoord); //Original Image
	
	#if Use_SurfaceBlur == 1
		float3 luma = (tex2D(GBlurSamplerPong,texcoord).rgb);
		//float3 luma = orig.rgb;
		float3 Color ;
		float3 factor;
		float3 Z = 0.0;
		float3 final_colour = 0.0;
	#else
		float4 color = tex2D(GBlurSamplerPong, texcoord) * sampleWeights[0];
	#endif
		[loop]
	for(int j = 1; j < N; ++j) {
		#if Use_SurfaceBlur == 1
			Color = tex2D(GBlurSamplerPong, texcoord + float2(0.0, sampleOffsets[j] * Gpx_size.y)).rgb;
			factor = normpdf4((Color-luma)) * sampleWeights[j];
			Z += factor;
			final_colour += factor*Color;
			Color = tex2D(GBlurSamplerPong, texcoord - float2(0.0, sampleOffsets[j] * Gpx_size.y)).rgb;
			factor = normpdf4((Color-luma)) * sampleWeights[j];
			Z += factor;
			final_colour += factor*Color;
		#else
			color += tex2D(GBlurSamplerPong, texcoord + float2(0.0, sampleOffsets[j] * Gpx_size.y)) * sampleWeights[j];
			color += tex2D(GBlurSamplerPong, texcoord - float2(0.0, sampleOffsets[j] * Gpx_size.y)) * sampleWeights[j];
		#endif
	}
	 
	#if Use_SurfaceBlur == 1
		float4 color = 0.0;
		color.rgb = (final_colour/Z);
	#endif
	
	orig = lerp(orig, color, BlurStrength);
	return saturate(orig);
}
#endif

#if Use_Unsharpmask == 1

#define UnsharpmaskSigma1 0.39894*exp(-0.5*0*0/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma2 0.39894*exp(-0.5*1*1/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma3 0.39894*exp(-0.5*2*2/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma4 0.39894*exp(-0.5*3*3/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma5 0.39894*exp(-0.5*4*4/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma

#if UnsharpmaskSigma >= 2
#define UnsharpmaskSigma6 0.39894*exp(-0.5*5*5/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma7 0.39894*exp(-0.5*6*6/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#endif

#if UnsharpmaskSigma >= 3
#define UnsharpmaskSigma8 0.39894*exp(-0.5*7*7/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma9 0.39894*exp(-0.5*8*8/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#endif 

#if UnsharpmaskSigma >= 4
#define UnsharpmaskSigma10 0.39894*exp(-0.5*9*9/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma11 0.39894*exp(-0.5*10*10/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma12 0.39894*exp(-0.5*11*11/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma13 0.39894*exp(-0.5*12*12/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#endif 

#if UnsharpmaskSigma >= 5
#define UnsharpmaskSigma14 0.39894*exp(-0.5*13*13/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma15 0.39894*exp(-0.5*14*14/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#endif

#if UnsharpmaskSigma >= 6
#define UnsharpmaskSigma16 0.39894*exp(-0.5*15*15/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma17 0.39894*exp(-0.5*17*17/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#endif 

#if UnsharpmaskSigma >= 8
#define UnsharpmaskSigma18 0.39894*exp(-0.5*17*17/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma19 0.39894*exp(-0.5*18*18/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#endif 

#if UnsharpmaskSigma >= 9
#define UnsharpmaskSigma20 0.39894*exp(-0.5*19*19/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#define UnsharpmaskSigma21 0.39894*exp(-0.5*20*20/(UnsharpmaskSigma*UnsharpmaskSigma))/UnsharpmaskSigma
#endif

#define UnsharpmaskWeight1 (UnsharpmaskSigma1)
#define UnsharpmaskOffset1 (0.0)

#define UnsharpmaskWeight2 (UnsharpmaskSigma2 + UnsharpmaskSigma3)
#define UnsharpmaskOffset2 ((UnsharpmaskSigma2*1.0)+(UnsharpmaskSigma3*2.0))/UnsharpmaskWeight2

#define UnsharpmaskWeight3 (UnsharpmaskSigma4 + UnsharpmaskSigma5 )
#define UnsharpmaskOffset3 ((UnsharpmaskSigma4*3.0)+(UnsharpmaskSigma5*4.0))/UnsharpmaskWeight3

#if UnsharpmaskSigma >= 2
#define UnsharpmaskWeight4 (UnsharpmaskSigma6 + UnsharpmaskSigma7)
#define UnsharpmaskOffset4 ((UnsharpmaskSigma6*5.0)+(UnsharpmaskSigma7*6.0))/UnsharpmaskWeight4
#else 
#define UnsharpmaskWeight4 1.0
#define UnsharpmaskOffset4 1.0
#endif

#if UnsharpmaskSigma >= 3
#define UnsharpmaskWeight5 (UnsharpmaskSigma8 + UnsharpmaskSigma9)
#define UnsharpmaskOffset5 ((UnsharpmaskSigma8*7.0)+(UnsharpmaskSigma9*8.0))/UnsharpmaskWeight5
#else 
#define UnsharpmaskWeight5 1.0
#define UnsharpmaskOffset5 1.0
#endif

#if UnsharpmaskSigma >= 4
#define UnsharpmaskWeight6 (UnsharpmaskSigma10 + UnsharpmaskSigma11)
#define UnsharpmaskOffset6 ((UnsharpmaskSigma10*9.0)+(UnsharpmaskSigma11*10.0))/UnsharpmaskWeight6

#define UnsharpmaskWeight7 (UnsharpmaskSigma12 + UnsharpmaskSigma13)
#define UnsharpmaskOffset7 ((UnsharpmaskSigma12*11.0)+(UnsharpmaskSigma13*12.0))/UnsharpmaskWeight7
#else 
#define UnsharpmaskWeight6 1.0
#define UnsharpmaskOffset6 1.0

#define UnsharpmaskWeight7 1.0
#define UnsharpmaskOffset7 1.0
#endif 

#if UnsharpmaskSigma >= 5
#define UnsharpmaskWeight8 (UnsharpmaskSigma14 + UnsharpmaskSigma15)
#define UnsharpmaskOffset8 ((UnsharpmaskSigma14*13.0)+(UnsharpmaskSigma15*14.0))/UnsharpmaskWeight8
#else 
#define UnsharpmaskWeight8 1.0
#define UnsharpmaskOffset8 1.0
#endif

#if UnsharpmaskSigma >= 6
#define UnsharpmaskWeight9 (UnsharpmaskSigma16 + UnsharpmaskSigma17)
#define UnsharpmaskOffset9 ((UnsharpmaskSigma16*15.0)+(UnsharpmaskSigma17*16.0))/UnsharpmaskWeight9
#else 
#define UnsharpmaskWeight9 1.0
#define UnsharpmaskOffset9 1.0
#endif

#if UnsharpmaskSigma >= 8
#define UnsharpmaskWeight10 (UnsharpmaskSigma18 + UnsharpmaskSigma19)
#define UnsharpmaskOffset10 ((UnsharpmaskSigma18*17.0)+(UnsharpmaskSigma19*18.0))/UnsharpmaskWeight10
#else 
#define UnsharpmaskWeight10 1.0
#define UnsharpmaskOffset10 1.0
#endif

#if UnsharpmaskSigma >= 9
#define UnsharpmaskWeight11 (UnsharpmaskSigma20 + UnsharpmaskSigma21)
#define UnsharpmaskOffset11 ((UnsharpmaskSigma20*19.0)+(UnsharpmaskSigma21*20.0))/UnsharpmaskWeight11
#else 
#define UnsharpmaskWeight11 1.0
#define UnsharpmaskOffset11 1.0
#endif

#if UnsharpmaskSigma == 3
	uniform const int S = 5;
#elif UnsharpmaskSigma == 4 
	uniform const int S = 7;
#elif UnsharpmaskSigma == 5 
	uniform const int S = 8;
#elif UnsharpmaskSigma == 6 
	uniform const int S = 9;
#elif UnsharpmaskSigma == 7 
	uniform const int S = 9;
#elif UnsharpmaskSigma == 8 
	uniform const int S = 10;
#elif UnsharpmaskSigma == 9 
	uniform const int S = 11;
#elif UnsharpmaskSigma == 10 
	uniform const int S = 11;
#elif UnsharpmaskSigma == 2 
	uniform const int S = 4;
#else 
	uniform const int S = 3;
#endif

float4 STexScale(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	return tex2D(ReShade::BackBuffer,texcoord);
}

float4 HUnsharpBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	#if SharpTexScale == 0
		#define SharpSampler ReShade::BackBuffer
	#else 
		#define SharpSampler SBlurSamplerPing
	#endif
	
	float sampleOffsets[11] = { UnsharpmaskOffset1, UnsharpmaskOffset2, UnsharpmaskOffset3, UnsharpmaskOffset4, UnsharpmaskOffset5, UnsharpmaskOffset6, UnsharpmaskOffset7, UnsharpmaskOffset8, UnsharpmaskOffset9, UnsharpmaskOffset10, UnsharpmaskOffset11 };
	float sampleWeights[11] = { UnsharpmaskWeight1, UnsharpmaskWeight2, UnsharpmaskWeight3, UnsharpmaskWeight4, UnsharpmaskWeight5, UnsharpmaskWeight6, UnsharpmaskWeight7, UnsharpmaskWeight8, UnsharpmaskWeight9, UnsharpmaskWeight10, UnsharpmaskWeight11 };
	
	float4 color = tex2D(SharpSampler, texcoord) * sampleWeights[0];
	
	[loop]
	for(int i = 1; i < S; ++i) {
		color += tex2D(SharpSampler, texcoord + float2(sampleOffsets[i] * Spx_size.x, 0.0)) * sampleWeights[i];
		color += tex2D(SharpSampler, texcoord - float2(sampleOffsets[i] * Spx_size.x, 0.0)) * sampleWeights[i]; 
	}

	return color;
}

float4 VUnsharpBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	
	float sampleOffsets[11] = { UnsharpmaskOffset1, UnsharpmaskOffset2, UnsharpmaskOffset3, UnsharpmaskOffset4, UnsharpmaskOffset5, UnsharpmaskOffset6, UnsharpmaskOffset7, UnsharpmaskOffset8, UnsharpmaskOffset9, UnsharpmaskOffset10, UnsharpmaskOffset11 };
	float sampleWeights[11] = { UnsharpmaskWeight1, UnsharpmaskWeight2, UnsharpmaskWeight3, UnsharpmaskWeight4, UnsharpmaskWeight5, UnsharpmaskWeight6, UnsharpmaskWeight7, UnsharpmaskWeight8, UnsharpmaskWeight9, UnsharpmaskWeight10, UnsharpmaskWeight11 };
	
	float4 color = tex2D(SBlurSamplerPong, texcoord) * sampleWeights[0];
	
	[loop]
	for(int i = 1; i < S; ++i) {
		color += tex2D(SBlurSamplerPong, texcoord + float2(0.0,sampleOffsets[i] * Spx_size.y)) * sampleWeights[i];
		color += tex2D(SBlurSamplerPong, texcoord - float2(0.0,sampleOffsets[i] * Spx_size.y)) * sampleWeights[i]; 
	}

	return color;
}

float4 H2UnsharpBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{

	float sampleOffsets[11] = { UnsharpmaskOffset1, UnsharpmaskOffset2, UnsharpmaskOffset3, UnsharpmaskOffset4, UnsharpmaskOffset5, UnsharpmaskOffset6, UnsharpmaskOffset7, UnsharpmaskOffset8, UnsharpmaskOffset9, UnsharpmaskOffset10, UnsharpmaskOffset11 };
	float sampleWeights[11] = { UnsharpmaskWeight1, UnsharpmaskWeight2, UnsharpmaskWeight3, UnsharpmaskWeight4, UnsharpmaskWeight5, UnsharpmaskWeight6, UnsharpmaskWeight7, UnsharpmaskWeight8, UnsharpmaskWeight9, UnsharpmaskWeight10, UnsharpmaskWeight11 };
	
	float4 color = tex2D(SBlurSamplerPing, texcoord) * sampleWeights[0];
	
	[loop]
	for(int i = 1; i < S; ++i) {
		color += tex2D(SBlurSamplerPing, texcoord + float2(sampleOffsets[i] * Spx_size.x, 0.0)) * sampleWeights[i];
		color += tex2D(SBlurSamplerPing, texcoord - float2(sampleOffsets[i] * Spx_size.x, 0.0)) * sampleWeights[i]; 
	}

	return color;
}

float4 UnsharpmaskFinalPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{

	float sampleOffsets[11] = { UnsharpmaskOffset1, UnsharpmaskOffset2, UnsharpmaskOffset3, UnsharpmaskOffset4, UnsharpmaskOffset5, UnsharpmaskOffset6, UnsharpmaskOffset7, UnsharpmaskOffset8, UnsharpmaskOffset9, UnsharpmaskOffset10, UnsharpmaskOffset11 };
	float sampleWeights[11] = { UnsharpmaskWeight1, UnsharpmaskWeight2, UnsharpmaskWeight3, UnsharpmaskWeight4, UnsharpmaskWeight5, UnsharpmaskWeight6, UnsharpmaskWeight7, UnsharpmaskWeight8, UnsharpmaskWeight9, UnsharpmaskWeight10, UnsharpmaskWeight11 };
	
	float4 color = tex2D(SBlurSamplerPong, texcoord) * sampleWeights[0];
		[loop]
	for(int j = 1; j < S; ++j) {
		color += tex2D(SBlurSamplerPong, texcoord + float2(0.0, sampleOffsets[j] * Spx_size.y)) * sampleWeights[j];
		color += tex2D(SBlurSamplerPong, texcoord - float2(0.0, sampleOffsets[j] * Spx_size.y)) * sampleWeights[j];
	}

	float4 orig = tex2D(ReShade::BackBuffer, texcoord); //Original Image
	
	float3 sharp;
	sharp = (orig.rgb) - color.rgb;
	float sharp_luma1 = dot(sharp, sharp_strength_luma_G);
	sharp_luma1 = clamp(sharp_luma1, -sharp_clampG, sharp_clampG);
	orig = orig + sharp_luma1;
	
	//orig = color;
	return saturate(orig);
}
#endif

#if Use_GaussianBloom == 1

#define BSigma1 0.39894*exp(-0.5*0*0/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma2 0.39894*exp(-0.5*1*1/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma3 0.39894*exp(-0.5*2*2/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma4 0.39894*exp(-0.5*3*3/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma5 0.39894*exp(-0.5*4*4/(BloomSigma*BloomSigma))/BloomSigma

#if BloomSigma >= 2
#define BSigma6 0.39894*exp(-0.5*5*5/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma7 0.39894*exp(-0.5*6*6/(BloomSigma*BloomSigma))/BloomSigma
#endif

#if BloomSigma >= 3
#define BSigma8 0.39894*exp(-0.5*7*7/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma9 0.39894*exp(-0.5*8*8/(BloomSigma*BloomSigma))/BloomSigma
#endif 

#if BloomSigma >= 4
#define BSigma10 0.39894*exp(-0.5*9*9/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma11 0.39894*exp(-0.5*10*10/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma12 0.39894*exp(-0.5*11*11/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma13 0.39894*exp(-0.5*12*12/(BloomSigma*BloomSigma))/BloomSigma
#endif 

#if BloomSigma >= 5
#define BSigma14 0.39894*exp(-0.5*13*13/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma15 0.39894*exp(-0.5*14*14/(BloomSigma*BloomSigma))/BloomSigma
#endif

#if BloomSigma >= 6
#define BSigma16 0.39894*exp(-0.5*15*15/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma17 0.39894*exp(-0.5*17*17/(BloomSigma*BloomSigma))/BloomSigma
#endif 

#if BloomSigma >= 8
#define BSigma18 0.39894*exp(-0.5*17*17/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma19 0.39894*exp(-0.5*18*18/(BloomSigma*BloomSigma))/BloomSigma
#endif 

#if BloomSigma >= 9
#define BSigma20 0.39894*exp(-0.5*19*19/(BloomSigma*BloomSigma))/BloomSigma
#define BSigma21 0.39894*exp(-0.5*20*20/(BloomSigma*BloomSigma))/BloomSigma
#endif

#define BloomWeight1 (BSigma1)
#define BloomOffset1 (0.0)

#define BloomWeight2 (BSigma2 + BSigma3)
#define BloomOffset2 ((BSigma2*1.0)+(BSigma3*2.0))/BloomWeight2

#define BloomWeight3 (BSigma4 + BSigma5 )
#define BloomOffset3 ((BSigma4*3.0)+(BSigma5*4.0))/BloomWeight3

#if BloomSigma >= 2
#define BloomWeight4 (BSigma6 + BSigma7)
#define BloomOffset4 ((BSigma6*5.0)+(BSigma7*6.0))/BloomWeight4
#else 
#define BloomWeight4 1.0
#define BloomOffset4 1.0
#endif

#if BloomSigma >= 3
#define BloomWeight5 (BSigma8 + BSigma9)
#define BloomOffset5 ((BSigma8*7.0)+(BSigma9*8.0))/BloomWeight5
#else 
#define BloomWeight5 1.0
#define BloomOffset5 1.0
#endif

#if BloomSigma >= 4
#define BloomWeight6 (BSigma10 + BSigma11)
#define BloomOffset6 ((BSigma10*9.0)+(BSigma11*10.0))/BloomWeight6

#define BloomWeight7 (BSigma12 + BSigma13)
#define BloomOffset7 ((BSigma12*11.0)+(BSigma13*12.0))/BloomWeight7
#else 
#define BloomWeight6 1.0
#define BloomOffset6 1.0

#define BloomWeight7 1.0
#define BloomOffset7 1.0
#endif 

#if BloomSigma >= 5
#define BloomWeight8 (BSigma14 + BSigma15)
#define BloomOffset8 ((BSigma14*13.0)+(BSigma15*14.0))/BloomWeight8
#else 
#define BloomWeight8 1.0
#define BloomOffset8 1.0
#endif

#if BloomSigma >= 6
#define BloomWeight9 (BSigma16 + BSigma17)
#define BloomOffset9 ((BSigma16*15.0)+(BSigma17*16.0))/BloomWeight9
#else 
#define BloomWeight9 1.0
#define BloomOffset9 1.0
#endif

#if BloomSigma >= 8
#define BloomWeight10 (BSigma18 + BSigma19)
#define BloomOffset10 ((BSigma18*17.0)+(BSigma19*18.0))/BloomWeight10
#else 
#define BloomWeight10 1.0
#define BloomOffset10 1.0
#endif

#if BloomSigma >= 9
#define BloomWeight11 (BSigma20 + BSigma21)
#define BloomOffset11 ((BSigma20*19.0)+(BSigma21*20.0))/BloomWeight11
#else 
#define BloomWeight11 1.0
#define BloomOffset11 1.0
#endif

#if BloomSigma == 3
	uniform const int B = 5;
#elif BloomSigma == 4 
	uniform const int B = 7;
#elif BloomSigma == 5 
	uniform const int B = 8;
#elif BloomSigma == 6 
	uniform const int B = 9;
#elif BloomSigma == 7 
	uniform const int B = 9;
#elif BloomSigma == 8 
	uniform const int B = 10;
#elif BloomSigma == 9 
	uniform const int B = 11;
#elif BloomSigma == 10 
	uniform const int B = 11;
#elif BloomSigma == 2 
	uniform const int B = 4;
#else 
	uniform const int B = 3;
#endif

float3 GlareDetectionPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	
	float3 color = tex2D( ReShade::BackBuffer, texcoord ).rgb;
	
	float3 orig = color;
	
		float2 offsets[5] = { 
        float2( 0.0,  0.0), 
        float2(-1.0,  0.0), 
        float2( 1.0,  0.0), 
        float2( 0.0, -1.0),
        float2( 0.0,  1.0),
							};
	
	float3 Mix = color;	
	[loop]
	for(int k = 1; k < 5; ++k) {
		Mix = max(tex2D(ReShade::BackBuffer, texcoord + offsets[k] * ReShade::PixelSize).rgb,Mix);
	}
	
	color.rgb *= float3(BloomRed,BloomGreen,BloomBlue);

	color.rgb = pow(abs(color.rgb),GaussExposure*0.01);
	Mix.rgb = pow(abs(Mix.rgb),GaussExposure*0.01);
	float mix = max(Mix.r, max(Mix.g, Mix.b));
	color.rgb = lerp(0.0,color.rgb,gDoSmoothererstep(GaussThreshold,1.0001,mix));
	
#if GaussBloomWarmth == 1
	float3 fringe = float3(color.r,1-pow(max(0,1-color.g),0.5),1-pow(max(0,1-color.b),0.25));
	color.rgb = fringe;
#elif GaussBloomWarmth == 3 
	float3 fringe = pow(max(0.0,color.rgb),8.0);
	fringe = max(pow(color.rgb*float3(0.0,1.0,1.0),2.0),pow(fringe,2.0));
	fringe = max(color*float3(0.0,0.0,1.0),fringe);
	fringe += pow(abs(color.rgb*float3(0.10,0.025,0.030)),0.35);
	color.rgb = pow(fringe,1.0);
#endif
	
	return color;
}

float4 HBloomBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{

	float sampleOffsets[11] = { BloomOffset1, BloomOffset2, BloomOffset3, BloomOffset4, BloomOffset5, BloomOffset6, BloomOffset7, BloomOffset8, BloomOffset9, BloomOffset10, BloomOffset11 };
	float sampleWeights[11] = { BloomWeight1, BloomWeight2, BloomWeight3, BloomWeight4, BloomWeight5, BloomWeight6, BloomWeight7, BloomWeight8, BloomWeight9, BloomWeight10, BloomWeight11 };
	
	float4 color = tex2D(BBlurSamplerPing, texcoord) * sampleWeights[0];
	[loop]
	for(int i = 1; i < B; ++i) {
		color += tex2D(BBlurSamplerPing, texcoord + float2(sampleOffsets[i] * ReShade::PixelSize.x * BloomOffset, 0.0)) * sampleWeights[i];
		color += tex2D(BBlurSamplerPing, texcoord - float2(sampleOffsets[i] * ReShade::PixelSize.x * BloomOffset, 0.0)) * sampleWeights[i]; 
	}
	
	return color;
}

float4 FinalBloomPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{

	float sampleOffsets[11] = { BloomOffset1, BloomOffset2, BloomOffset3, BloomOffset4, BloomOffset5, BloomOffset6, BloomOffset7, BloomOffset8, BloomOffset9, BloomOffset10, BloomOffset11 };
	float sampleWeights[11] = { BloomWeight1, BloomWeight2, BloomWeight3, BloomWeight4, BloomWeight5, BloomWeight6, BloomWeight7, BloomWeight8, BloomWeight9, BloomWeight10, BloomWeight11 };
	
	float4 color = tex2D(BBlurSamplerPong, texcoord) * sampleWeights[0];

	[loop]
	for(int j = 1; j < B; ++j) {
		color += tex2D(BBlurSamplerPong, texcoord + float2(0.0, sampleOffsets[j] * ReShade::PixelSize.y * BloomOffset)) * sampleWeights[j];
		color += tex2D(BBlurSamplerPong, texcoord - float2(0.0, sampleOffsets[j] * ReShade::PixelSize.y * BloomOffset)) * sampleWeights[j];
	}

	float4 orig = tex2D(ReShade::BackBuffer, texcoord); //Original Image
	//color.rgb = 1-pow(max(0,1-color.rgb),2);
	
	color.rgb = lerp(dot(color.rgb,0.333),color.rgb,BloomSaturation); 
	
			#define BloomDebug 0
		
		#if (GaussBloomWarmth == 0)
			#if BloomDebug == 1 
			orig = (1.0 - ((0.95 - orig) * (1.05 - (pow(abs(color*2.6),2.5))*0.3)));
			#else
			color.rgb = 1-pow(max(0,1-color.rgb),2);
			color = pow(abs(1.05*orig*color),0.75);
			orig = lerp(orig, (1.0 - ((1.0 - orig) * (1.00 - (color)))), BloomStrength);
			#endif
		#elif (GaussBloomWarmth == 1)
			#if BloomDebug == 1 
			orig = (1.0 - ((1.05 - orig*0.8) * (0.95 - (pow(abs(color*2.6),2.5))*0.3)));
			#else
			color.rgb = 1-pow(max(0,1-color.rgb),2.0);
			orig = lerp(orig, (1.0 - ((1.0 - orig) * (1.0 - (color)))), BloomStrength);
			#endif
		#elif (GaussBloomWarmth == 3)
			#if BloomDebug == 1 
			orig = (1.0 - ((1.05 - orig*0.8) * (0.95 - (pow(abs(color*2.6),2.5))*0.3)));
			#else
			color.rgb = 1-pow(max(0,1-color.rgb),2.0);
			orig = lerp(orig, (1.0 - ((1.0 - orig) * (1.0 - (color)))), BloomStrength);
			#endif
		#else
			#if BloomDebug == 1 
			orig = (1.0 - ((1.0 - orig) * (1.0 - (color))));
			#else
			color.rgb = 1-pow(max(0,1-color.rgb),2.5);
			orig = lerp(orig, (1.0 - ((1.0 - orig) * (1.0 - (color)))), BloomStrength); 
			#endif
		#endif 
		
	return saturate(orig);
}
#if GaussBloomQuality >= 1
float4 V2BloomBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	/*
	#if BloomSigma == 0
	float sampleOffsets[5] = { 0.0, 1.44, 3.36, 5.28, 7.2 };
	float sampleWeights[5] = { 0.1611802578, 0.2656817436, 0.1217707992, 0.0286519528, 0.0031667948 };
	int N =  5;
	#elif BloomSigma == 1
	float sampleOffsets[6] = { 0.0, 2.4594594594, 4.4054054051, 6.3513513514, 8.2972972993, 10.2432432612 };
	float sampleWeights[6] = { 0.1320605996, 0.2314535771, 0.1362670411, 0.0518407222, 0.0124417733, 0.0018103109 };
	int N =  6;
	#elif BloomSigma == 2
	float sampleOffsets[9] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087 };
	float sampleWeights[9] = { 0.1097184698841, 0.11815835218275, 0.10971846988355, 0.087774775907115, 0.0603451584361505, 0.035497152021265, 0.017748576010605, 0.0074730846360555, 0.0026155796226175 };
	int N = 9;
	#elif BloomSigma == 3
	float sampleOffsets[14] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045 };
	float sampleWeights[14] = { 0.07217787046621, 0.0739383063308, 0.07217787046621, 0.0671422050846, 0.0595124090525, 0.050254923194, 0.0404224382261, 0.030961867593, 0.0225763617753, 0.015665230617, 0.01033905220881, 0.006487248442, 0.0038673981121, 0.0021890932698 };
	int N = 14;
	#elif BloomSigma == 4
	float sampleOffsets[49] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045, 26.304348, 28.2173915, 30.130435, 32.0434785,  33.956522, 35.8695655, 37.782609, 39.6956525, 41.608696, 43.5217395, 45.434783, 47.3478265, 49.26087, 51.1739135, 53.086957, 55.0000005, 56.913044, 58.8260875, 60.739131, 62.6521745, 62.6521745, 66.4782615, 68.391305, 70.3043485, 72.217392, 74.1304355, 76.043479, 77.9565225, 79.869566, 81.7826095, 83.695653, 85.6086965, 87.52174, 89.4347835, 91.347827 };
	float sampleWeights[27] = { 0.038588721157425, 0.04019658453898,  0.0393926528482025, 0.038588721157425, 0.0370754379748375, 0.03556215479225, 0.033510492016125, 0.03145882924, 0.02908457797625, 0.0267103267125, 0.02423714831875, 0.021763969925, 0.01938971865625, 0.0170154673875, 0.01488853396875, 0.01276160055, 0.0109704987125, 0.009179396875, 0.0077550077065, 0.006330618538, 0.00525763234525, 0.0041846461525, 0.0034174610244375, 0.002650275896375, 0.0021289101456875, 0.001607544395, 0.001270478635 };
	int N = 27;
	#elif BloomSigma == 5
	float sampleOffsets[49] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045, 26.304348, 28.2173915, 30.130435, 32.0434785,  33.956522, 35.8695655, 37.782609, 39.6956525, 41.608696, 43.5217395, 45.434783, 47.3478265, 49.26087, 51.1739135, 53.086957, 55.0000005, 56.913044, 58.8260875, 60.739131, 62.6521745, 62.6521745, 66.4782615, 68.391305, 70.3043485, 72.217392, 74.1304355, 76.043479, 77.9565225, 79.869566, 81.7826095, 83.695653, 85.6086965, 87.52174, 89.4347835, 91.347827 };
	float sampleWeights[53] = { 0.0254526381164209635, 0.025709735471131608,  0.0254526381164209635, 0.025195540761710319, 0.0249384434069996745, 0.02468134605228903, 0.0241973980904975475, 0.023713450128706065, 0.0232295021669145825, 0.0227455542051231, 0.022089432449318325, 0.02143331069351355, 0.020777188937708775, 0.020121067181904, 0.01936178162775675, 0.0186024960736095, 0.01784321051946225, 0.017083924965315, 0.01629300251499375, 0.0155020800646725, 0.01471115761435125, 0.01392023516403, 0.01316094960828375, 0.0124016640525375, 0.01164237849679125, 0.010883092941045, 0.01020289963372875, 0.0095227063264125, 0.00884251301909625, 0.00816231971178, 0.0075895253441475, 0.007016730976515, 0.0064439366088825, 0.00587114224125, 0.0054156225851637, 0.0049601029290774, 0.0045045832729911, 0.0040490636169048, 0.00370592263246335, 0.0033627816480219, 0.00301964066358045, 0.002676499679139, 0.0024311538751846125, 0.002185808071230225, 0.0019404622672758375, 0.00169511646332145, 0.0015283836962515875, 0.001361650929181725, 0.0011949181621118625, 0.001028185395042, 0.000920391764994, 0.000812598134946 };
	int N = 49;
	#endif
	*/
	
	float sampleOffsets[11] = { BloomOffset1, BloomOffset2, BloomOffset3, BloomOffset4, BloomOffset5, BloomOffset6, BloomOffset7, BloomOffset8, BloomOffset9, BloomOffset10, BloomOffset11 };
	float sampleWeights[11] = { BloomWeight1, BloomWeight2, BloomWeight3, BloomWeight4, BloomWeight5, BloomWeight6, BloomWeight7, BloomWeight8, BloomWeight9, BloomWeight10, BloomWeight11 };
	
	float4 color = tex2D(BBlurSamplerPong, texcoord) * sampleWeights[0];

	[loop]
	for(int j = 1; j < B; ++j) {
		color += tex2D(BBlurSamplerPong, texcoord + float2(0.0, sampleOffsets[j] * ReShade::PixelSize.y * BloomOffset)) * sampleWeights[j];
		color += tex2D(BBlurSamplerPong, texcoord - float2(0.0, sampleOffsets[j] * ReShade::PixelSize.y * BloomOffset)) * sampleWeights[j];
	}

	return color;
}

float4 H2BloomBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	/*
	#if BloomSigma == 0
	float sampleOffsets[5] = { 0.0, 1.44, 3.36, 5.28, 7.2 };
	float sampleWeights[5] = { 0.1611802578, 0.2656817436, 0.1217707992, 0.0286519528, 0.0031667948 };
	int N =  5;
	#elif BloomSigma == 1
	float sampleOffsets[6] = { 0.0, 2.4594594594, 4.4054054051, 6.3513513514, 8.2972972993, 10.2432432612 };
	float sampleWeights[6] = { 0.1320605996, 0.2314535771, 0.1362670411, 0.0518407222, 0.0124417733, 0.0018103109 };
	int N =  6;
	#elif BloomSigma == 2
	float sampleOffsets[9] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087 };
	float sampleWeights[9] = { 0.1097184698841, 0.11815835218275, 0.10971846988355, 0.087774775907115, 0.0603451584361505, 0.035497152021265, 0.017748576010605, 0.0074730846360555, 0.0026155796226175 };
	int N = 9;
	#elif BloomSigma == 3
	float sampleOffsets[14] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045 };
	float sampleWeights[14] = { 0.07217787046621, 0.0739383063308, 0.07217787046621, 0.0671422050846, 0.0595124090525, 0.050254923194, 0.0404224382261, 0.030961867593, 0.0225763617753, 0.015665230617, 0.01033905220881, 0.006487248442, 0.0038673981121, 0.0021890932698 };
	int N = 14;
	#elif BloomSigma == 4
	float sampleOffsets[49] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045, 26.304348, 28.2173915, 30.130435, 32.0434785,  33.956522, 35.8695655, 37.782609, 39.6956525, 41.608696, 43.5217395, 45.434783, 47.3478265, 49.26087, 51.1739135, 53.086957, 55.0000005, 56.913044, 58.8260875, 60.739131, 62.6521745, 62.6521745, 66.4782615, 68.391305, 70.3043485, 72.217392, 74.1304355, 76.043479, 77.9565225, 79.869566, 81.7826095, 83.695653, 85.6086965, 87.52174, 89.4347835, 91.347827 };
	float sampleWeights[27] = { 0.038588721157425, 0.04019658453898,  0.0393926528482025, 0.038588721157425, 0.0370754379748375, 0.03556215479225, 0.033510492016125, 0.03145882924, 0.02908457797625, 0.0267103267125, 0.02423714831875, 0.021763969925, 0.01938971865625, 0.0170154673875, 0.01488853396875, 0.01276160055, 0.0109704987125, 0.009179396875, 0.0077550077065, 0.006330618538, 0.00525763234525, 0.0041846461525, 0.0034174610244375, 0.002650275896375, 0.0021289101456875, 0.001607544395, 0.001270478635 };
	int N = 27;
	#elif BloomSigma == 5
	float sampleOffsets[49] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045, 26.304348, 28.2173915, 30.130435, 32.0434785,  33.956522, 35.8695655, 37.782609, 39.6956525, 41.608696, 43.5217395, 45.434783, 47.3478265, 49.26087, 51.1739135, 53.086957, 55.0000005, 56.913044, 58.8260875, 60.739131, 62.6521745, 62.6521745, 66.4782615, 68.391305, 70.3043485, 72.217392, 74.1304355, 76.043479, 77.9565225, 79.869566, 81.7826095, 83.695653, 85.6086965, 87.52174, 89.4347835, 91.347827 };
	float sampleWeights[53] = { 0.0254526381164209635, 0.025709735471131608,  0.0254526381164209635, 0.025195540761710319, 0.0249384434069996745, 0.02468134605228903, 0.0241973980904975475, 0.023713450128706065, 0.0232295021669145825, 0.0227455542051231, 0.022089432449318325, 0.02143331069351355, 0.020777188937708775, 0.020121067181904, 0.01936178162775675, 0.0186024960736095, 0.01784321051946225, 0.017083924965315, 0.01629300251499375, 0.0155020800646725, 0.01471115761435125, 0.01392023516403, 0.01316094960828375, 0.0124016640525375, 0.01164237849679125, 0.010883092941045, 0.01020289963372875, 0.0095227063264125, 0.00884251301909625, 0.00816231971178, 0.0075895253441475, 0.007016730976515, 0.0064439366088825, 0.00587114224125, 0.0054156225851637, 0.0049601029290774, 0.0045045832729911, 0.0040490636169048, 0.00370592263246335, 0.0033627816480219, 0.00301964066358045, 0.002676499679139, 0.0024311538751846125, 0.002185808071230225, 0.0019404622672758375, 0.00169511646332145, 0.0015283836962515875, 0.001361650929181725, 0.0011949181621118625, 0.001028185395042, 0.000920391764994, 0.000812598134946 };
	int N = 49;
	#endif
	*/
	
	float sampleOffsets[11] = { BloomOffset1, BloomOffset2, BloomOffset3, BloomOffset4, BloomOffset5, BloomOffset6, BloomOffset7, BloomOffset8, BloomOffset9, BloomOffset10, BloomOffset11 };
	float sampleWeights[11] = { BloomWeight1, BloomWeight2, BloomWeight3, BloomWeight4, BloomWeight5, BloomWeight6, BloomWeight7, BloomWeight8, BloomWeight9, BloomWeight10, BloomWeight11 };
	
	float4 color = tex2D(BBlurSamplerPing, texcoord) * sampleWeights[0];

	[loop]
	for(int j = 1; j < B; ++j) {
		color += tex2D(BBlurSamplerPing, texcoord + float2(sampleOffsets[j] * ReShade::PixelSize.x * BloomOffset,0.0)) * sampleWeights[j];
		color += tex2D(BBlurSamplerPing, texcoord - float2(sampleOffsets[j] * ReShade::PixelSize.x * BloomOffset,0.0)) * sampleWeights[j];
	}
	
	return color;
}
#endif
#if GaussBloomQuality >= 2
float4 V3BloomBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	/*
	#if BloomSigma == 0
	float sampleOffsets[5] = { 0.0, 1.44, 3.36, 5.28, 7.2 };
	float sampleWeights[5] = { 0.1611802578, 0.2656817436, 0.1217707992, 0.0286519528, 0.0031667948 };
	int N =  5;
	#elif BloomSigma == 1
	float sampleOffsets[6] = { 0.0, 2.4594594594, 4.4054054051, 6.3513513514, 8.2972972993, 10.2432432612 };
	float sampleWeights[6] = { 0.1320605996, 0.2314535771, 0.1362670411, 0.0518407222, 0.0124417733, 0.0018103109 };
	int N =  6;
	#elif BloomSigma == 2
	float sampleOffsets[9] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087 };
	float sampleWeights[9] = { 0.1097184698841, 0.11815835218275, 0.10971846988355, 0.087774775907115, 0.0603451584361505, 0.035497152021265, 0.017748576010605, 0.0074730846360555, 0.0026155796226175 };
	int N = 9;
	#elif BloomSigma == 3
	float sampleOffsets[14] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045 };
	float sampleWeights[14] = { 0.07217787046621, 0.0739383063308, 0.07217787046621, 0.0671422050846, 0.0595124090525, 0.050254923194, 0.0404224382261, 0.030961867593, 0.0225763617753, 0.015665230617, 0.01033905220881, 0.006487248442, 0.0038673981121, 0.0021890932698 };
	int N = 14;
	#elif BloomSigma == 4
	float sampleOffsets[49] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045, 26.304348, 28.2173915, 30.130435, 32.0434785,  33.956522, 35.8695655, 37.782609, 39.6956525, 41.608696, 43.5217395, 45.434783, 47.3478265, 49.26087, 51.1739135, 53.086957, 55.0000005, 56.913044, 58.8260875, 60.739131, 62.6521745, 62.6521745, 66.4782615, 68.391305, 70.3043485, 72.217392, 74.1304355, 76.043479, 77.9565225, 79.869566, 81.7826095, 83.695653, 85.6086965, 87.52174, 89.4347835, 91.347827 };
	float sampleWeights[27] = { 0.038588721157425, 0.04019658453898,  0.0393926528482025, 0.038588721157425, 0.0370754379748375, 0.03556215479225, 0.033510492016125, 0.03145882924, 0.02908457797625, 0.0267103267125, 0.02423714831875, 0.021763969925, 0.01938971865625, 0.0170154673875, 0.01488853396875, 0.01276160055, 0.0109704987125, 0.009179396875, 0.0077550077065, 0.006330618538, 0.00525763234525, 0.0041846461525, 0.0034174610244375, 0.002650275896375, 0.0021289101456875, 0.001607544395, 0.001270478635 };
	int N = 27;
	#elif BloomSigma == 5
	float sampleOffsets[49] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045, 26.304348, 28.2173915, 30.130435, 32.0434785,  33.956522, 35.8695655, 37.782609, 39.6956525, 41.608696, 43.5217395, 45.434783, 47.3478265, 49.26087, 51.1739135, 53.086957, 55.0000005, 56.913044, 58.8260875, 60.739131, 62.6521745, 62.6521745, 66.4782615, 68.391305, 70.3043485, 72.217392, 74.1304355, 76.043479, 77.9565225, 79.869566, 81.7826095, 83.695653, 85.6086965, 87.52174, 89.4347835, 91.347827 };
	float sampleWeights[53] = { 0.0254526381164209635, 0.025709735471131608,  0.0254526381164209635, 0.025195540761710319, 0.0249384434069996745, 0.02468134605228903, 0.0241973980904975475, 0.023713450128706065, 0.0232295021669145825, 0.0227455542051231, 0.022089432449318325, 0.02143331069351355, 0.020777188937708775, 0.020121067181904, 0.01936178162775675, 0.0186024960736095, 0.01784321051946225, 0.017083924965315, 0.01629300251499375, 0.0155020800646725, 0.01471115761435125, 0.01392023516403, 0.01316094960828375, 0.0124016640525375, 0.01164237849679125, 0.010883092941045, 0.01020289963372875, 0.0095227063264125, 0.00884251301909625, 0.00816231971178, 0.0075895253441475, 0.007016730976515, 0.0064439366088825, 0.00587114224125, 0.0054156225851637, 0.0049601029290774, 0.0045045832729911, 0.0040490636169048, 0.00370592263246335, 0.0033627816480219, 0.00301964066358045, 0.002676499679139, 0.0024311538751846125, 0.002185808071230225, 0.0019404622672758375, 0.00169511646332145, 0.0015283836962515875, 0.001361650929181725, 0.0011949181621118625, 0.001028185395042, 0.000920391764994, 0.000812598134946 };
	int N = 49;
	#endif
	*/
	
	float sampleOffsets[11] = { BloomOffset1, BloomOffset2, BloomOffset3, BloomOffset4, BloomOffset5, BloomOffset6, BloomOffset7, BloomOffset8, BloomOffset9, BloomOffset10, BloomOffset11 };
	float sampleWeights[11] = { BloomWeight1, BloomWeight2, BloomWeight3, BloomWeight4, BloomWeight5, BloomWeight6, BloomWeight7, BloomWeight8, BloomWeight9, BloomWeight10, BloomWeight11 };
	
	float4 color = tex2D(BBlurSamplerPong, texcoord) * sampleWeights[0];

	[loop]
	for(int j = 1; j < B; ++j) {
		color += tex2D(BBlurSamplerScalePong, texcoord + float2(0.0, sampleOffsets[j] * ReShade::PixelSize.y * BloomOffset)) * sampleWeights[j];
		color += tex2D(BBlurSamplerScalePong, texcoord - float2(0.0, sampleOffsets[j] * ReShade::PixelSize.y * BloomOffset)) * sampleWeights[j];
	}

	return color;
}

float4 H3BloomBlurPS(in float4 pos : SV_Position, in float2 texcoord : TEXCOORD) : COLOR
{
	/*
	#if BloomSigma == 0
	float sampleOffsets[5] = { 0.0, 1.44, 3.36, 5.28, 7.2 };
	float sampleWeights[5] = { 0.1611802578, 0.2656817436, 0.1217707992, 0.0286519528, 0.0031667948 };
	int N =  5;
	#elif BloomSigma == 1
	float sampleOffsets[6] = { 0.0, 2.4594594594, 4.4054054051, 6.3513513514, 8.2972972993, 10.2432432612 };
	float sampleWeights[6] = { 0.1320605996, 0.2314535771, 0.1362670411, 0.0518407222, 0.0124417733, 0.0018103109 };
	int N =  6;
	#elif BloomSigma == 2
	float sampleOffsets[9] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087 };
	float sampleWeights[9] = { 0.1097184698841, 0.11815835218275, 0.10971846988355, 0.087774775907115, 0.0603451584361505, 0.035497152021265, 0.017748576010605, 0.0074730846360555, 0.0026155796226175 };
	int N = 9;
	#elif BloomSigma == 3
	float sampleOffsets[14] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045 };
	float sampleWeights[14] = { 0.07217787046621, 0.0739383063308, 0.07217787046621, 0.0671422050846, 0.0595124090525, 0.050254923194, 0.0404224382261, 0.030961867593, 0.0225763617753, 0.015665230617, 0.01033905220881, 0.006487248442, 0.0038673981121, 0.0021890932698 };
	int N = 14;
	#elif BloomSigma == 4
	float sampleOffsets[49] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045, 26.304348, 28.2173915, 30.130435, 32.0434785,  33.956522, 35.8695655, 37.782609, 39.6956525, 41.608696, 43.5217395, 45.434783, 47.3478265, 49.26087, 51.1739135, 53.086957, 55.0000005, 56.913044, 58.8260875, 60.739131, 62.6521745, 62.6521745, 66.4782615, 68.391305, 70.3043485, 72.217392, 74.1304355, 76.043479, 77.9565225, 79.869566, 81.7826095, 83.695653, 85.6086965, 87.52174, 89.4347835, 91.347827 };
	float sampleWeights[27] = { 0.038588721157425, 0.04019658453898,  0.0393926528482025, 0.038588721157425, 0.0370754379748375, 0.03556215479225, 0.033510492016125, 0.03145882924, 0.02908457797625, 0.0267103267125, 0.02423714831875, 0.021763969925, 0.01938971865625, 0.0170154673875, 0.01488853396875, 0.01276160055, 0.0109704987125, 0.009179396875, 0.0077550077065, 0.006330618538, 0.00525763234525, 0.0041846461525, 0.0034174610244375, 0.002650275896375, 0.0021289101456875, 0.001607544395, 0.001270478635 };
	int N = 27;
	#elif BloomSigma == 5
	float sampleOffsets[49] = { 0.0, 1.4347826, 3.3478260, 5.2608695, 7.1739130, 9.0869565, 11, 12.9130435, 14.826087, 16.7391305, 18.652174, 20.5652175, 22.478261, 24.3913045, 26.304348, 28.2173915, 30.130435, 32.0434785,  33.956522, 35.8695655, 37.782609, 39.6956525, 41.608696, 43.5217395, 45.434783, 47.3478265, 49.26087, 51.1739135, 53.086957, 55.0000005, 56.913044, 58.8260875, 60.739131, 62.6521745, 62.6521745, 66.4782615, 68.391305, 70.3043485, 72.217392, 74.1304355, 76.043479, 77.9565225, 79.869566, 81.7826095, 83.695653, 85.6086965, 87.52174, 89.4347835, 91.347827 };
	float sampleWeights[53] = { 0.0254526381164209635, 0.025709735471131608,  0.0254526381164209635, 0.025195540761710319, 0.0249384434069996745, 0.02468134605228903, 0.0241973980904975475, 0.023713450128706065, 0.0232295021669145825, 0.0227455542051231, 0.022089432449318325, 0.02143331069351355, 0.020777188937708775, 0.020121067181904, 0.01936178162775675, 0.0186024960736095, 0.01784321051946225, 0.017083924965315, 0.01629300251499375, 0.0155020800646725, 0.01471115761435125, 0.01392023516403, 0.01316094960828375, 0.0124016640525375, 0.01164237849679125, 0.010883092941045, 0.01020289963372875, 0.0095227063264125, 0.00884251301909625, 0.00816231971178, 0.0075895253441475, 0.007016730976515, 0.0064439366088825, 0.00587114224125, 0.0054156225851637, 0.0049601029290774, 0.0045045832729911, 0.0040490636169048, 0.00370592263246335, 0.0033627816480219, 0.00301964066358045, 0.002676499679139, 0.0024311538751846125, 0.002185808071230225, 0.0019404622672758375, 0.00169511646332145, 0.0015283836962515875, 0.001361650929181725, 0.0011949181621118625, 0.001028185395042, 0.000920391764994, 0.000812598134946 };
	int N = 49;
	#endif
	*/
	
	float sampleOffsets[11] = { BloomOffset1, BloomOffset2, BloomOffset3, BloomOffset4, BloomOffset5, BloomOffset6, BloomOffset7, BloomOffset8, BloomOffset9, BloomOffset10, BloomOffset11 };
	float sampleWeights[11] = { BloomWeight1, BloomWeight2, BloomWeight3, BloomWeight4, BloomWeight5, BloomWeight6, BloomWeight7, BloomWeight8, BloomWeight9, BloomWeight10, BloomWeight11 };
	
	float4 color = tex2D(BBlurSamplerPing, texcoord) * sampleWeights[0];

	[loop]
	for(int j = 1; j < B; ++j) {
		color += tex2D(BBlurSamplerScalePing, texcoord + float2(sampleOffsets[j] * ReShade::PixelSize.x * BloomOffset,0.0)) * sampleWeights[j];
		color += tex2D(BBlurSamplerScalePing, texcoord - float2(sampleOffsets[j] * ReShade::PixelSize.x * BloomOffset,0.0)) * sampleWeights[j];
	}
	
	return color;
}
#endif
#endif

technique Gaussian_Tech <bool enabled = RESHADE_START_ENABLED; int toggle = Gaussian_ToggleKey; >
{

#if Use_Unsharpmask == 1
#if SharpTexScale != 0
	pass H1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = STexScale;
		RenderTarget = SBlurTex2Dping;
	}
#endif
	pass H1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = HUnsharpBlurPS;
		RenderTarget = SBlurTex2Dpong;
	}
#if UnsharpmaskQuality >= 1
	pass V1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = VUnsharpBlurPS;
		RenderTarget = SBlurTex2Dping;
	}
	pass H2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H2UnsharpBlurPS;
		RenderTarget = SBlurTex2Dpong;
	}
#endif
	
	pass VFinal
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = UnsharpmaskFinalPS;
	}
#endif
#if Use_GaussianBloom == 1
	
	pass GD
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = GlareDetectionPS;
		RenderTarget = BBlurTex2Dping;
	}

	pass H1Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = HBloomBlurPS;
		RenderTarget = BBlurTex2Dpong;
	}
	
#if GaussBloomQuality >= 1
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V2BloomBlurPS;
		RenderTarget = BBlurTex2Dping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H2BloomBlurPS;
		#if GaussBloomQuality >= 2
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif	

#if GaussBloomQuality >= 2
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V2BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 3
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif

#if GaussBloomQuality >= 3
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V3BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 4
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif	

#if GaussBloomQuality >= 4
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V3BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 5
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif

#if GaussBloomQuality >= 5
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V3BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 6
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif

#if GaussBloomQuality >= 6
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V3BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 7
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif

#if GaussBloomQuality >= 7
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V3BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 8
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif

#if GaussBloomQuality >= 8
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V3BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 9
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif

#if GaussBloomQuality >= 9
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V3BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 10
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif

#if GaussBloomQuality >= 10
	pass V2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = V3BloomBlurPS;
		RenderTarget = BBlurTexScaleping;
	}
	
	pass H2Bloom
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H3BloomBlurPS;
		#if GaussBloomQuality >= 11
			RenderTarget = BBlurTexScalepong;
		#else
			RenderTarget = BBlurTex2Dpong;
		#endif
	}
#endif
	
	pass VFinal
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = FinalBloomPS;
	}
#endif

#if Use_GaussianBlur == 1
#if GaussTexScale != 0
	pass H1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = BlurTexScale;
		RenderTarget = GBlurTex2Dping;
	}
#endif	
	pass H1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = HGaussianBlurPS;
		RenderTarget = GBlurTex2Dpong;
	}
#if GaussQuality >= 1
	pass V1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = VGaussianBlurPS;
		RenderTarget = GBlurTex2Dping;
	}
	pass H2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H2GaussianBlurPS;
		RenderTarget = GBlurTex2Dpong;
	}
#endif
#if GaussQuality >= 2
	pass V1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = VGaussianBlurPS;
		RenderTarget = GBlurTex2Dping;
	}
	pass H2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H2GaussianBlurPS;
		RenderTarget = GBlurTex2Dpong;
	}
#endif
#if GaussQuality >= 3
	pass V1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = VGaussianBlurPS;
		RenderTarget = GBlurTex2Dping;
	}
	pass H2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H2GaussianBlurPS;
		RenderTarget = GBlurTex2Dpong;
	}
#endif
#if GaussQuality >= 4
	pass V1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = VGaussianBlurPS;
		RenderTarget = GBlurTex2Dping;
	}
	pass H2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H2GaussianBlurPS;
		RenderTarget = GBlurTex2Dpong;
	}
#endif
#if GaussQuality >= 5
	pass V1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = VGaussianBlurPS;
		RenderTarget = GBlurTex2Dping;
	}
	pass H2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H2GaussianBlurPS;
		RenderTarget = GBlurTex2Dpong;
	}
#endif
#if GaussQuality >= 6
	pass V1
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = VGaussianBlurPS;
		RenderTarget = GBlurTex2Dping;
	}
	pass H2
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = H2GaussianBlurPS;
		RenderTarget = GBlurTex2Dpong;
	}
#endif
	pass VFinal
	{
		VertexShader = ReShade::VS_PostProcess;
		PixelShader = GaussianBlurFinalPS;
	}
#endif
}

#endif

#include EFFECT_CONFIG_UNDEF(Ioxa)
